/*-
 * ============LICENSE_START=======================================================
 * ONAP : CCSDK
 * ================================================================================
 * Copyright (C) 2017 AT&T Intellectual Property. All rights
 * 						reserved.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ============LICENSE_END=========================================================
 */

package org.onap.ccsdk.sli.core.sli.provider;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.onap.ccsdk.sli.core.sli.SvcLogicContext;
import org.onap.ccsdk.sli.core.sli.SvcLogicException;
import org.onap.ccsdk.sli.core.sli.SvcLogicExpression;
import org.onap.ccsdk.sli.core.sli.SvcLogicNode;
import org.onap.ccsdk.sli.core.sli.SvcLogicResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class UpdateNodeExecutor extends SvcLogicNodeExecutor {

	private static final Logger LOG = LoggerFactory
			.getLogger(UpdateNodeExecutor.class);

	@Override
	public SvcLogicNode execute(SvcLogicServiceImpl svc, SvcLogicNode node,
			SvcLogicContext ctx) throws SvcLogicException {

		String plugin = SvcLogicExpressionResolver.evaluate(
				node.getAttribute("plugin"), node, ctx);
		String resourceType = SvcLogicExpressionResolver.evaluate(
				node.getAttribute("resource"), node, ctx);
		String key = SvcLogicExpressionResolver.evaluateAsKey(
				node.getAttribute("key"), node, ctx);
		String pfx = SvcLogicExpressionResolver.evaluate(
				node.getAttribute("pfx"), node, ctx);


		Map<String, String> parmMap = new HashMap<String, String>();

		Set<Map.Entry<String, SvcLogicExpression>> parmSet = node
				.getParameterSet();
		boolean hasParms = false;

		for (Iterator<Map.Entry<String, SvcLogicExpression>> iter = parmSet
				.iterator(); iter.hasNext();) {
			hasParms = true;
			Map.Entry<String, SvcLogicExpression> curEnt = iter.next();
			String curName = curEnt.getKey();
			SvcLogicExpression curExpr = curEnt.getValue();
			if (curExpr != null) {
				String curExprValue = SvcLogicExpressionResolver.evaluate(
						curExpr, node, ctx);

				LOG.debug("Parameter " + curName + " = "
						+ curExpr.asParsedExpr() + " resolves to "
						+ curExprValue);

				parmMap.put(curName, curExprValue);
			}
		}

		String outValue = "failure";

		if (LOG.isDebugEnabled()) {
			LOG.debug("save node encountered - looking for resource class "
					+ plugin);
		}


        SvcLogicResource resourcePlugin = getSvcLogicResource(plugin);


			if (resourcePlugin != null) {

				try {
					switch (resourcePlugin.update(resourceType, key,
							parmMap, pfx, ctx)) {
					case SUCCESS:
						outValue = "success";
						break;
					case NOT_FOUND:
						outValue = "not-found";
						break;
					case FAILURE:
					default:
						outValue = "failure";
					}
				} catch (SvcLogicException e) {
					LOG.error("Caught exception from resource plugin", e);
					outValue = "failure";
				}
			} else {
				LOG.warn("Could not find SvcLogicResource object for plugin "
						+ plugin);
			}

		SvcLogicNode nextNode = node.getOutcomeValue(outValue);
		if (nextNode != null) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("about to execute " + outValue + " branch");
			}
			return (nextNode);
		}

		nextNode = node.getOutcomeValue("Other");
		if (nextNode != null) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("about to execute Other branch");
			}
		} else {
			if (LOG.isDebugEnabled()) {
				LOG.debug("no "+outValue+" or Other branch found");
			}
		}
		return (nextNode);
	}

}
